/***************************************************************************
 *   Copyright (C) 2007-2010 by Tobias Schwarz                             *
 *   mail@tobias-schwarz.net                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 *   In addition, as a special exception, the copyright holders give       *
 *   permission to link the code of this program with any edition of       *
 *   the Qt library by Trolltech AS, Norway (or with modified versions     *
 *   of Qt that use the same license as Qt), and distribute linked         *
 *   combinations including the two.  You must obey the GNU General        *
 *   Public License in all respects for all of the code used other than    *
 *   Qt.  If you modify this file, you may extend this exception to        *
 *   your version of the file, but you are not obligated to do so.  If     *
 *   you do not wish to do so, delete this exception statement from        *
 *   your version.                                                         *
 ***************************************************************************/

#ifndef HIGHLIGHTER_H
#define HIGHLIGHTER_H

#include <QSyntaxHighlighter>

#include <QHash>
#include <QTextCharFormat>

class QTextDocument;

class Highlighter : public QSyntaxHighlighter
{
Q_OBJECT

public:
   Highlighter(QTextDocument *parent = 0);
   void setSyntax(const int newSyntax);

protected:
   void highlightBlock(const QString &text);

private:
   void defineFormats();
   void highlightAASequence(const QString &text, const int start, const int end);
   void highlightNucSequence(const QString &text, const int start, const int end);
   int syntax;
   struct HighlightingRule
   {
      int type; // 0 = general, 1 = Any sequence, 2 = AA sequence, 3 = Nuc. Sequence
      QRegExp pattern;
      QTextCharFormat format;
   };

   QRegExp AAStart;
   QRegExp AAEnd;

   QVector<HighlightingRule> FASTARules;
   QVector<HighlightingRule> EMBLRules;
   QVector<HighlightingRule> AlignmentRules;
   QVector<HighlightingRule> GbRules;

   QVector<HighlightingRule> aaSequenceRules;
   QVector<HighlightingRule> nucSequenceRules;

   QTextCharFormat seq_name;
   QTextCharFormat comment;
   QTextCharFormat cystein;
   QTextCharFormat basic_aa;
   QTextCharFormat dibasic_aa;
   QTextCharFormat acidic_aa;
   QTextCharFormat polar_aa;
   QTextCharFormat apolar_aa;
   QTextCharFormat a_nuc;
   QTextCharFormat t_nuc;
   QTextCharFormat g_nuc;
   QTextCharFormat c_nuc;
};

#endif
